package jdk.management.resource.internal.inst;

import jdk.internal.instrumentation.Logger;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.jar.JarFile;
import jdk.internal.instrumentation.InstrumentationTarget;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.HashSet;
import java.io.IOException;
import java.util.stream.Collectors;
import java.util.jar.JarEntry;
import java.util.Iterator;
import java.util.Set;
import jdk.internal.instrumentation.ClassInstrumentation;
import java.util.List;
import java.util.ArrayList;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.nio.file.CopyOption;
import java.util.HashMap;
import java.io.File;

public final class StaticInstrumentation
{
    public StaticInstrumentation() {
    }
    
    public static void main(String[] array) throws Exception {
        instrumentClassesForResourceManagement(new File(array[0]), new File(array[1]));
    }
    
    public static void instrumentClassesForResourceManagement(File obj, File file) throws Exception {
        if (!obj.isDirectory()) {
            throw new Exception(obj + " is not a directory");
        }
        if (!file.isDirectory()) {
            throw new Exception(obj + " is not a directory");
        }
        StaticInstrumentation.InstrumentationLogger instrumentationLogger = new StaticInstrumentation.InstrumentationLogger();
        System.out.println();
        System.out.println("Reading from " + obj);
        System.out.println("Output to " + file);
        Set<File> allJarFiles = findAllJarFiles(obj);
        HashMap<File, List<Class>> hashMap = new HashMap();
        System.out.println();
        System.out.println("Searching for classes");
        int i = 0;
        Class[] hooks = InitInstrumentation.hooks;
        int length = hooks.length;
        int j = 0;
        while (j < length) {
            Class<Object> clazz = hooks[j];
            String targetClassName = findTargetClassName(clazz);
            System.out.println(i + ":");
            ++i;
            System.out.println("   Instrumentation: " + clazz.getName());
            System.out.println("   Target         : " + targetClassName);
            boolean n = false;
            for (File file2 : allJarFiles) {
                JarEntry jarEntry = getJarEntry(targetClassName, file2);
                if (jarEntry != null) {
                    System.out.println("   Found in jar  : " + file2);
                    if (jarEntry.getCodeSigners() != null) {
                        throw new Exception("The target class '" + targetClassName + "' was found in a signed jar: " + file2);
                    }
                    addNewTask(hashMap, file2, clazz);
                    n = true;
                    break;
                }
                else {
                    continue;
                }
            }
            if (n == 0) {
                throw new Exception("The target class '" + targetClassName + " was not found in any jar");
            }
            ++j;
        }
        System.out.println();
        System.out.println("Instrumenting");
        for (File file3 : hashMap.keySet()) {
            File obj2 = new File(file, file3.getName());
            Files.copy(file3.toPath(), obj2.toPath(), new CopyOption[] { StandardCopyOption.REPLACE_EXISTING });
            System.out.println("   Jar     : " + file3);
            System.out.println("   Jar copy: " + obj2);
            ArrayList o = new ArrayList();
            for (Class clazz2 : (List)hashMap.get(file3)) {
                String targetClassName2 = findTargetClassName(clazz2);
                System.out.println("      Class: " + targetClassName2);
                byte[] newBytes = new ClassInstrumentation(clazz2, targetClassName2, findSourceBytesFor(targetClassName2, obj2), instrumentationLogger).getNewBytes();
                File outputFile = createOutputFile(file, targetClassName2);
                writeOutputClass(outputFile, newBytes);
                o.add(outputFile);
            }
            System.out.println("   Updating jar");
            updateJar(file, obj2, o);
            System.out.println();
        }
        return;
    }
    
    private static void updateJar(File directory, File file, List<File> list) throws InterruptedException, IOException {
        ProcessBuilder processBuilder = new ProcessBuilder(new String[] { System.getProperty("java.home") + File.separator + "bin" + File.separator + "jar", "uvf", file.getAbsolutePath() });
        Iterator<Object> iterator = list.iterator();
        while (iterator.hasNext()) {
            processBuilder.command().add(directory.toPath().relativize(((File)iterator.next()).toPath()).toString());
        }
        processBuilder.directory(directory);
        processBuilder.redirectOutput(ProcessBuilder.Redirect.INHERIT);
        processBuilder.redirectError(ProcessBuilder.Redirect.INHERIT);
        System.out.println("Executing: " + (String)processBuilder.command().stream().collect(Collectors.joining(" ")));
        processBuilder.start().waitFor();
    }
    
    private static void addNewTask(HashMap<File, List<Class<?>>> hashMap, File file, Class<?> clazz) {
        List<Class<?>> value = (List)hashMap.get(file);
        if (value == null) {
            value = new ArrayList();
            hashMap.put(file, value);
        }
        value.add(clazz);
    }
    
    private static Set<File> findAllJarFiles(File file) throws IOException {
        HashSet<File> set = new HashSet();
        LinkedBlockingDeque<File> linkedBlockingDeque = new LinkedBlockingDeque();
        linkedBlockingDeque.add(file);
        File file2;
        while ((file2 = (File)linkedBlockingDeque.poll()) != null) {
            for (File file3 : file2.listFiles()) {
                if (file3.isDirectory()) {
                    linkedBlockingDeque.add(file3);
                }
                else if (file3.getName().endsWith(".jar")) {
                    set.add(file3);
                }
            }
        }
        return set;
    }
    
    private static File createOutputFile(File parent, String s) {
        File file = new File(parent, s.replace(".", File.separator) + ".class");
        file.getParentFile().mkdirs();
        return file;
    }
    
    private static void writeOutputClass(File file, byte[] b) throws FileNotFoundException, IOException {
        try (final FileOutputStream fileOutputStream = new FileOutputStream(file)) {
            fileOutputStream.write(b);
        }
    }
    
    private static String findTargetClassName(Class<?> clazz) {
        return ((InstrumentationTarget)clazz.getAnnotation(InstrumentationTarget.class)).value();
    }
    
    private static JarEntry getJarEntry(String s, File file) throws Exception {
        try (final JarFile jarFile = new JarFile(file)) {
            return jarEntry = jarFile.getJarEntry(s.replace(".", "/") + ".class");
        }
    }
    
    private static byte[] findSourceBytesFor(String s, File file) throws Exception {
        try (final JarFile jarFile = new JarFile(file)) {
            ZipEntry entry = jarFile.getEntry(s.replace(".", "/") + ".class");
            if (entry == null) {
                return null;
            }
            else {
                return bytes = readBytes(jarFile.getInputStream(entry));
            }
        }
    }
    
    private static byte[] readBytes(InputStream inputStream) throws IOException {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        byte[] array = new byte[1024];
        int read;
        while ((read = inputStream.read(array)) != -1) {
            byteArrayOutputStream.write(array, 0, read);
        }
        return byteArrayOutputStream.toByteArray();
    }
    
    static class InstrumentationLogger implements Logger
    {
        public void error(String str) {
            System.err.println("StaticInstrumentation error: " + str);
        }
        
        public void warn(String str) {
            System.err.println("StaticInstrumentation warning: " + str);
        }
        
        public void info(String str) {
            System.err.println("StaticInstrumentation info: " + str);
        }
        
        public void debug(String s) {
        }
        
        public void trace(String s) {
        }
        
        public void error(String str, Throwable obj) {
            System.err.println("StaticInstrumentation error: " + str + ": " + obj);
        }
    }
}
